module PhracBridge (

  run

) where

import Phrac.ParseSyntax hiding ( Info )
import Phrac.Driver
import Phrac.Interpreter
import Phrac.Configuration
import qualified Phrac.Error as PE

import Logging
import Pretty
import MosmlPrelude
import Translation ( coreId2VarId )
import Identifiers ( CoreId(..) )

defaultFlags = [TerminationCondition AllowDiverge]

flags Nothing = defaultFlags
flags (Just Debug) = DebugLevel PE.Debug : defaultFlags
flags (Just Info) = DebugLevel PE.Verbose : defaultFlags
flags (Just Warn) = defaultFlags
flags (Just Fatal) = defaultFlags
flags (Just Ignore) = defaultFlags
    

run :: Maybe LogLevel -> Program -> IO ()
run ll (Program tds clss insts vals) = 
    do let myflags = flags ll
       vals' <- genPhracPrelude myflags
       x <- startFromAST myflags (Program tds clss insts (vals'++vals))
       case x of
         Left err ->
             fatal ("generated program does not type check: " ++ err)
         Right prog' ->
             return ()

genPhracPrelude :: [Flag] -> IO [ValBind]
genPhracPrelude flags = 
    do let src = map ( \(x,e) -> coreId2VarId (CoreId x) ++ " = " ++ e ++ ";") 
                 phracPrelude
       res <- srcToAST flags (unlines src)
       case res of
         Left error ->
             fatal ("parsing of ML prelude for PHRaC failed: " ++ error)
         Right (Program [] [] [] vs) ->
             return vs
         Right p ->
             fatal ("no type definitions, classes, or instances in ML prelude " ++
                    "for PHRaC expected")
    