module SemanticObjectTest where

import TestFramework

import Text.ParserCombinators.Parsec
import SemanticObjectParser
import SemanticObject
import Identifiers

structureSig1 = "- > New type names: t   \t\n" ++
                "  \t structure X : {type t = t, val x : t}"

structureSig2 = "- > New type names: t/1, s " ++
                " structure Y : {type t = t/1, type s = s -> int}"

structureSig3 = "- > structure Z : {type t = int, val 'a 'b x : 'a -> int}"

functorSig1 = 
    "- > functor F : " ++
    "!t s.{type t = t, type s = s}->?t/1 s/1.{type t = t/1, type s = s/1}"

functorSig2 = 
    "- > functor G : {val x : int}->{val x : int}"

functorSig3 = 
    "- > functor H : {}->{ }"

functorSig4 = 
    "- > functor F : {val x : int}->!t.{type t = t}->{val x : int}"

functorSig5 = 
    "- > functor F : !t.{type t = t}->!t/1.{type t = t/1}->{type t = int}"

parseString :: SemParser a -> String -> IO a
parseString p s = 
    case runParser p emptySemState "" s of
      Left err -> do assertFailure ("unexpected parse error:\ninput: " ++ s ++
                                    "\nerror message: " ++ show err)
                     fail "error"
      Right x -> return x

$(tests "semanticObjectParserTests" [d|

 testStructure1 = 
     do (name, sig) <- parseString structureSig structureSig1
        assertEqual "X" name 
        assertEqual (SemExStr [SemTyvar "t"] 
                     (SemStr 
                      [(TypId "t", SemSimTypTyvar (SemTyvar "t"))]
                      [(ValId "x", SemValTyp [] $ SemSimTypTyvar (SemTyvar "t"))]))
                    sig

 testStructure2 = 
     do (name, sig) <- parseString structureSig structureSig2
        assertEqual "Y" name 
        assertEqual sig (SemExStr [SemTyvar "t/1", SemTyvar "s"] 
                         (SemStr 
                          [(TypId "t", SemSimTypTyvar (SemTyvar "t/1")),
                           (TypId "s", SemTycon (SemTyconId "->")
                                       [SemSimTypTyvar (SemTyvar "s"),
                                        SemTycon (SemTyconId "int") []])]
                          []))

 testStructure3 = 
     do (name, sig) <- parseString structureSig structureSig3
        assertEqual "Z" name 
        assertEqual (SemExStr [] 
                     (SemStr 
                      [(TypId "t", SemTycon (SemTyconId "int") [])]
                      [(ValId "x", SemValTyp [SemSimTyvar "'a"] $
                                   SemTycon (SemTyconId "->")
                                   [SemSimTypSimTyvar (SemSimTyvar "'a"),
                                    SemTycon (SemTyconId "int") []])]))
                    sig

 testEmptyStructure = 
    do (name, sig) <- parseString structureSig "- > structure A : {}"
       assertEqual "A" name
       assertEqual (SemExStr [] (SemStr [] [])) sig

 testFunctor1 = 
     do (name, sig) <- parseString functorSig functorSig1
        assertEqual "F" name
        assertEqual (SemFun
                     [SemTyvar "t", SemTyvar "s"]
                     [SemStr [(TypId "t", SemSimTypTyvar $ SemTyvar "t"),
                              (TypId "s", SemSimTypTyvar $ SemTyvar "s")] []]
                     (SemExStr [SemTyvar "t/1", SemTyvar "s/1"]
                      (SemStr [(TypId "t", SemSimTypTyvar $ SemTyvar "t/1"),
                               (TypId "s", SemSimTypTyvar $ SemTyvar "s/1")] [])))
                    sig

 testFunctor2 = 
     do (name, sig) <- parseString functorSig functorSig2
        assertEqual "G" name
        assertEqual (SemFun
                     []
                     [SemStr [] 
                      [(ValId "x", SemValTyp [] (SemTycon intTycon []))]]
                     (SemExStr []
                      (SemStr [] 
                       [(ValId "x", SemValTyp [] (SemTycon intTycon []))])))
                    sig

 testFunctor3 = 
     do (name, sig) <- parseString functorSig functorSig3
        assertEqual "H" name
        assertEqual (SemFun
                     []
                     [SemStr [] []]
                     (SemExStr [] (SemStr [] [])))
                    sig

 testFunctor4 = 
     do (name, sig) <- parseString functorSig functorSig4
        assertEqual "F" name
        assertEqual (SemFun
                     [SemTyvar "t"]
                     [ SemStr [] 
                       [(ValId "x", SemValTyp [] (SemTycon intTycon []))]
                     , SemStr 
                       [(TypId "t", SemSimTypTyvar $ SemTyvar "t")] []]
                     (SemExStr []
                      (SemStr [] 
                       [(ValId "x", SemValTyp [] (SemTycon intTycon []))])))
                    sig

 testFunctor5 = 
     do (name, sig) <- parseString functorSig functorSig5
        assertEqual "F" name
        assertEqual (SemFun
                     [SemTyvar "t", SemTyvar "t/1"]
                     [ SemStr [(TypId "t", SemSimTypTyvar $ SemTyvar "t")] []
                     , SemStr [(TypId "t", SemSimTypTyvar $ SemTyvar "t/1")] []]
                     (SemExStr []
                      (SemStr [(TypId "t", SemTycon intTycon [])] [])))
                    sig

 |])